@@ -2,10 +2,10 @@
 ## This is used in 'pomp.R' and 'builder.R'.
 
 workhorse_templates <- list(
-  initializer=list(
-    slotname="initializer",
+  rinit=list(
+    slotname="rinit",
     Cname="__pomp_rinit",
-    proto=quote(initializer(params,t0,...)),
+    proto=quote(rinit(...)),
     header="\nvoid __pomp_rinit (double *__x, const double *__p, double t, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars)\n{\n",
     footer="\n}\n\n",
     vars=list(
@@ -26,8 +26,8 @@ workhorse_templates <- list(
   rmeasure=list(
     slotname="rmeasure",
     Cname="__pomp_rmeasure",
-    proto=quote(rmeasure(x,t,params,...)),
-    header="\nvoid __pomp_rmeasure (double *__y, const double *__x, const double *__p, const int *__obsindex, const int *__stateindex, const int *__parindex, const int *__covindex, int __ncovars, const double *__covars, double t)\n{\n",
+    proto=quote(rmeasure(...)),
+    header="\nvoid __pomp_rmeasure (double *__y, const double *__x, const double *__p, const int *__obsindex, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars, double t)\n{\n",
     footer="\n}\n\n",
     vars=list(
       params=list(
@@ -51,8 +51,8 @@ workhorse_templates <- list(
   dmeasure=list(
     slotname="dmeasure",
     Cname= "__pomp_dmeasure",
-    proto=quote(dmeasure(y,x,t,params,log,...)),
-    header="\nvoid __pomp_dmeasure (double *__lik, const double *__y, const double *__x, const double *__p, int give_log, const int *__obsindex, const int *__stateindex, const int *__parindex, const int *__covindex, int __ncovars, const double *__covars, double t)\n{\n",
+    proto=quote(dmeasure(log,...)),
+    header="\nvoid __pomp_dmeasure (double *__lik, const double *__y, const double *__x, const double *__p, int give_log, const int *__obsindex, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars, double t)\n{\n",
     footer="\n}\n\n",
     vars=list(
       params=list(
@@ -80,8 +80,8 @@ workhorse_templates <- list(
   step.fn=list(
     slotname="step.fun",
     Cname="__pomp_stepfn",
-    proto=quote(step.fun(x,t,params,delta.t,...)),
-    header="\nvoid __pomp_stepfn (double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, int __covdim, const double *__covars, double t, double dt)\n{\n",
+    proto=quote(step.fun(...)),
+    header="\nvoid __pomp_stepfn (double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars, double t, double dt)\n{\n",
     footer="\n}\n\n",
     vars=list(
       params=list(
@@ -101,8 +101,8 @@ workhorse_templates <- list(
   rate.fn=list(
     slotname="rate.fun",
     Cname="__pomp_ratefn",
-    proto=quote(rate.fun(j,x,t,params,...)),
-    header="\ndouble __pomp_ratefn (int j, double t, double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, int __covdim, const double *__covars)\n{\n  double rate = 0.0;  \n",
+    proto=quote(rate.fun(j,...)),
+    header="\ndouble __pomp_ratefn (int j, double t, double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars)\n{\n  double rate = 0.0;  \n",
     footer="  return rate;\n}\n\n",
     vars=list(
       params=list(
@@ -122,8 +122,8 @@ workhorse_templates <- list(
   dprocess=list(
     slotname="dprocess",
     Cname="__pomp_dproc",
-    proto=quote(dprocess(x1,x2,t1,t2,params,...)),
-    header="\nvoid __pomp_dproc (double *__loglik, const double *__x1, const double *__x2, double t_1, double t_2, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, int __ncovars, const double *__covars)\n{\n",
+    proto=quote(dprocess(...)),
+    header="\nvoid __pomp_dproc (double *__loglik, const double *__x1, const double *__x2, double t_1, double t_2, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars)\n{\n",
     footer="\n}\n\n",
     vars=list(
       params=list(
@@ -151,8 +151,8 @@ workhorse_templates <- list(
   skeleton=list(
     slotname="skeleton",
     Cname="__pomp_skelfn",
-    proto=quote(skeleton(x,t,params,...)),
-    header="\nvoid __pomp_skelfn (double *__f, const double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, int __ncovars, const double *__covars, double t)\n{\n",
+    proto=quote(skeleton(...)),
+    header="\nvoid __pomp_skelfn (double *__f, const double *__x, const double *__p, const int *__stateindex, const int *__parindex, const int *__covindex, const double *__covars, double t)\n{\n",
     footer="\n}\n\n",
     vars=list(
       params=list(
@@ -173,11 +173,11 @@ workhorse_templates <- list(
       )
     )
   ),
-  fromEstimationScale=list(
-    slotname="fromEstimationScale",
+  fromEst=list(
+    slotname="fromEst",
     Cname="__pomp_from_trans",
-    proto=quote(from.trans(params,...)),
-    header="\nvoid __pomp_from_trans (double *__pt, const double *__p, const int *__parindex)\n{\n",
+    proto=quote(from.trans(...)),
+    header="\nvoid __pomp_from_trans (double *__p, const double *__pt, const int *__parindex)\n{\n",
     footer="\n}\n\n",
     vars=list(
       covars=list(
@@ -189,15 +189,15 @@ workhorse_templates <- list(
         cref="__p[__parindex[{%v%}]]"
       ),
       transforms=list(
-        names=quote(paste0("T",paramnames)),
+        names=quote(paste0("T_",paramnames)),
         cref="__pt[__parindex[{%v%}]]"
       )
     )
   ),
-  toEstimationScale=list(
-    slotname="toEstimationScale",
+  toEst=list(
+    slotname="toEst",
     Cname="__pomp_to_trans",
-    proto=quote(to.trans(params,...)),
+    proto=quote(to.trans(...)),
     header="\nvoid __pomp_to_trans (double *__pt, const double *__p, const int *__parindex)\n{\n",
     footer="\n}\n\n",
     vars=list(
@@ -210,7 +210,7 @@ workhorse_templates <- list(
         cref="__p[__parindex[{%v%}]]"
       ),
       transforms=list(
-        names=quote(paste0("T",paramnames)),
+        names=quote(paste0("T_",paramnames)),
         cref="__pt[__parindex[{%v%}]]"
       )
     )
@@ -218,14 +218,10 @@ workhorse_templates <- list(
   rprior=list(
     slotname="rprior",
     Cname="__pomp_rprior",
-    proto=quote(rprior(params,...)),
+    proto=quote(rprior(...)),
     header="\nvoid __pomp_rprior (double *__p, const int *__parindex)\n{\n",
     footer="\n}\n\n",
     vars=list(
-      covars=list(
-        names=quote(covarnames),
-        cref="__covars[__covindex[{%v%}]]"
-      ),
       params=list(
         names=quote(paramnames),
         cref="__p[__parindex[{%v%}]]"
@@ -235,14 +231,10 @@ workhorse_templates <- list(
   dprior=list(
     slotname="dprior",
     Cname="__pomp_dprior",
-    proto=quote(dprior(params,log,...)),
+    proto=quote(dprior(log,...)),
     header="\nvoid __pomp_dprior (double *__lik, const double *__p, int give_log, const int *__parindex)\n{\n",
     footer="\n}\n\n",
     vars=list(
-      covars=list(
-        names=quote(covarnames),
-        cref="__covars[__covindex[{%v%}]]"
-      ),
       params=list(
         names=quote(paramnames),
         cref="__p[__parindex[{%v%}]]"
