@@ -74,7 +74,7 @@ void _ou2_pdf (double *f,
   const double *covars)
 {
   if (t2-t1 != 1)
-    errorcall(R_NilValue,"ou2_pdf error: transitions must be consecutive");
+    err("ou2_pdf error: transitions must be consecutive");
   f[0] = dens_ou2(x[X1],x[X2],z[X1],z[X2],ALPHA1,ALPHA2,ALPHA3,ALPHA4,
     SIGMA1,SIGMA2,SIGMA3,1);
 }
